<?php

namespace Drupal\parameter_pollution\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Handles HTTP Parameter Pollution by cleaning duplicate query parameters.
 *
 * Instead of rejecting the request, this subscriber removes any duplicate
 * query parameters (keeping only the last occurrence) and redirects to the
 * sanitized URL.
 */
class HppSubscriber implements EventSubscriberInterface {

  /**
   * Inspects incoming GET requests for duplicate query parameters.
   *
   * If duplicates are detected, rebuilds the query string so that only the
   * last occurrence of each parameter is kept, then redirects to the new URL.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The event to process.
   */
  public function onKernelRequest(RequestEvent $event) {
    $request = $event->getRequest();

    // Only process GET requests.
    if (!$request->isMethod('GET')) {
      return;
    }

    // Retrieve the raw query string.
    $queryString = $request->server->get('QUERY_STRING');
    if (empty($queryString)) {
      return;
    }

    // Explode the query string into key-value pairs.
    $pairs = explode('&', $queryString);
    $params = [];
    $hasDuplicates = FALSE;

    foreach ($pairs as $pair) {
      if ($pair === '') {
        continue;
      }
      // Split key and value (limit to 2 parts).
      $parts = explode('=', $pair, 2);
      $key = urldecode($parts[0]);
      // Use an empty string if no value is provided.
      $value = $parts[1] ?? '';

      // Check if the key has already been set.
      if (array_key_exists($key, $params)) {
        $hasDuplicates = TRUE;
      }
      // Overwrite the key so that only the last occurrence remains.
      $params[$key] = $value;
    }

    // If duplicates were found, rebuild the query string.
    if ($hasDuplicates) {
      // Build a new query string using the last occurrences.
      $newQueryString = UrlHelper::buildQuery($params);

      // Rebuild the full URL.
      $schemeAndHost = $request->getSchemeAndHttpHost();
      $path = $request->getPathInfo();
      $newUrl = $schemeAndHost . $path;
      if (!empty($newQueryString)) {
        $newUrl .= '?' . $newQueryString;
      }

      // Redirect to the sanitized URL.
      $response = new RedirectResponse($newUrl);
      $event->setResponse($response);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Priority 100 ensures this subscriber runs early.
    return [
      KernelEvents::REQUEST => ['onKernelRequest', 100],
    ];
  }

}
